// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author jarroyo
*/

//--------------------------------------------------------------------------------
// used libraries (#uses)



//--------------------------------------------------------------------------------
// variables and constants
const int iIEC104_IMPORT_TYPE         =  1;
const int iIEC104_IMPORT_NAME         =  2;
const int iIEC104_IMPORT_APPLICATION  =  3;
const int iIEC104_IMPORT_LOCAL_LIST   =  4;
const int iIEC104_IMPORT_GLOBAL_LIST  =  5;
const int iIEC104_IMPORT_ACTIVE       =  6;
const int iIEC104_IMPORT_ENCRYPTION   =  7;
const int iIEC104_IMPORT_CHANGE_ROLE  =  8;
const int iIEC104_IMPORT_INVALID_BIT  =  9;
const int iIEC104_IMPORT_FORCE_ACTIVE = 10;
const int iIEC104_IMPORT_CONNECTIONS  = 11;
const int iIEC104_IMPORT_LENGTH       = 11;


const string c_unSystemIntegrity_IEC104 = "IEC104_";  // Cconstant for IEC104 unicos front end system integrity


const string sIEC104_OBJ_CONN_LOCAL_LIST    = "LocalList";
const string sIEC104_OBJ_CONN_GLOBAL_LIST   = "GlobalList";
const string sIEC104_OBJ_CONN_ACTIVE        = "Active";
const string sIEC104_OBJ_CONN_ENCRYPTION    = "Encryption";
const string sIEC104_OBJ_CONN_CHANGE_ROLE   = "ChangeRole";
const string sIEC104_OBJ_CONN_INVALID_BIT   = "SetInvalidBit";
const string sIEC104_OBJ_CONN_FORCE_ACTIVE  = "ForceActive";
const string sIEC104_OBJ_CONN_CONN_HOSTS    = "ConnectionHosts";
const string sIEC104_OBJ_CONN_CONN_PORTS    = "ConnectionPorts";
const string sIEC104_OBJ_CONN_CONN_TIME_OUT = "ConnectionTimeOut";
const string sIEC104_OBJ_CONN_CONN_ACTIVE   = "ConnectionActive";


const int iIEC104_ADDRESS_REFERENCE = 1;
const int iIEC104_ADDRESS_SUBINDEX  = 2;
const int iIEC104_ADDRESS_DATATYPE  = 3;
const int iIEC104_ADDRESS_DIRECTION = 4;
const int iIEC104_ADDRESS_COT       = 5;
const int iIEC104_ADDRESS_LENGTH    = 5;


//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------


//------------------------------------------------------------------------------------------------------------------------
//unConfigIEC104_check()
/**
Purpose: check the coherency of the IEC-104 front-end config

Parameters:
  * sCurrentObject:   string,     input,  type of config
  * dsFrontEndLine:   dyn_string, input,  config line
  * dsDeleteDps:      dyn_string, input,  dp to be deleted
  * exceptionInfo:    dyn_string, output, variable to return errors

*/
void unConfigIEC104_check(string sCurrentObject,
                          dyn_string dsFrontEndLine,
                          dyn_string dsDeleteDps,
                          dyn_string &exceptionInfo)
{

  switch( sCurrentObject )
  {
    case UN_PLC_COMMAND:
      _unConfigIEC104_checkApplication(dsFrontEndLine, dsDeleteDps, exceptionInfo);
      break;

    default:
      fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration", "UNKNOWNFUNCTION"), "");
      return;
      break;
  }

}//  unConfigIEC104_check()





/**
Purpose: set OPC front-end config

Parameters:
  sCurrentObject:    string,     input,    Type of object command
  dsLine:            dyn_sring,  input,    Front end config line
  exceptionInfo:     dyn_string, output,   Exception output variable

Return: nothing
*/
void unConfigIEC104_set(string sCurrentObject, dyn_string dsLine, dyn_string &exceptionInfo)
{

  switch( sCurrentObject )
  {
    case UN_PLC_COMMAND:
      _unConfigIEC104_setApplication(dsLine, exceptionInfo);
      if( dynlen(exceptionInfo) > 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_set() -> Error setting the front end, due to: " + exceptionInfo[2], "");
        return;
      }
      break;


    default:
      fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_set() -> Unknown object: " + sCurrentObject);
      return;
      break;
  }

}//  unConfigIEC104_set()





/**
Purpose: check data for a sub application (not comm)

Parameters:
    * dsFrontEndLine: dyn_string, input, IEC-104 configuration line in file + additional parameters. It must contains:
        01. Front-end type
        02. Front-end name
        03. Front-end application
        04. IEC Connection - Local list
        05. IEC Connection - Global list
        06. IEC Connection - Active
        07. IEC Connection - Encryption
        08. IEC Connection - Change Role
        09. IEC Connection - Set invalid bit
        10. IEC Connection - Force host connection (0 = Auto, 1 = 1st host, 2 = 2nd host, ...)
        11. IEC Connections: Host IP address A | Host IP port A | Timeout [seconds] A, IP address B | Host IP port B | Timeout [seconds], ...

        (additional parameters)
        01. Driver number
        02. Boolean archive name
        03. Analog archive name
        04. Event16 bits
        05. Event archive name


    * dsDeleteDps: dyn_string, input, datapoints that will be deleted before importation

    * exceptionInfo: dyn_string, output, for errors

*/
void _unConfigIEC104_checkApplication(dyn_string dsFrontEndLine, dyn_string dsDeleteDps, dyn_string &exceptionInfo)
{
  bool bAreTheSame;
  int iLoop, iRes, iDriverNumber, iLen;
  string sFrontEndName, sApplication, sFrontEndDp, sFrontEndDpFromAlias, sFrontEndAlias, sFrontEndIecConnection, sHostName;
  dyn_string dsLine, dsLineAdditionalData, dsFrontEndIecConnections;
  dyn_errClass deError;
  mapping mIecConnection, mIecConnectionFromFile;


  iLen = dynlen(dsFrontEndLine);
  if( iLen == (iIEC104_IMPORT_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH) )
  {
    // Split FrontEnd line between IEC104 fields, and UNICOS additional fields
    dsLine = dsFrontEndLine;
    for( iLoop = 1 ; iLoop <= UN_CONFIG_PLC_ADDITIONAL_LENGTH ; iLoop++ )
    {
      dynAppend(dsLineAdditionalData, dsLine[iIEC104_IMPORT_LENGTH + 1]);
      dynRemove(dsLine,                      iIEC104_IMPORT_LENGTH + 1);
    }

    // Checking front end parameters
    _unConfigIEC104_checkData(dsLine, exceptionInfo);
    if( dynlen(exceptionInfo) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error checking front end parameters due to: " + exceptionInfo[2], "");
      return;
    }

    // Checking archive parameters
    _unConfigIEC104_checkAdditionalData(dsLineAdditionalData, exceptionInfo);
    if( dynlen(exceptionInfo) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error checking front end archive parameters due to: " + exceptionInfo[2], "");
      return;
    }


    // Check front end name and application name without spaces
    sFrontEndName = strrtrim(dsLine[UN_CONFIG_PLC_NAME]);
    sApplication  = strrtrim(dsLine[UN_CONFIG_PLC_SUBAPPLICATION]);


    // Check the system integrity name for the front-end
    sFrontEndDp          = getSystemName() + c_unSystemIntegrity_IEC104 + sFrontEndName;
    sFrontEndDpFromAlias = unGenericDpFunctions_dpAliasToName(sFrontEndName);


    // Remove possible "." in the DP name
    if( substr(sFrontEndDpFromAlias, strlen(sFrontEndDpFromAlias) - 1) == "." )
    {
      sFrontEndDpFromAlias = substr(sFrontEndDpFromAlias, 0, strlen(sFrontEndDpFromAlias) - 1);
    }


    // Check if DP extracted from the alias already exists
    if( (sFrontEndDpFromAlias != "")          &&
        (sFrontEndDpFromAlias != sFrontEndDp)   )
    {
      if( dynContains(dsDeleteDps, sFrontEndDpFromAlias) <= 0 )
      {
        // Alias exists, us not used by IEC104 config DP and will not be deleted
        fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> " + sFrontEndName + getCatStr("unGeneration", "BADHOSTNAMEALIAS") + sFrontEndDpFromAlias, "");
        return;
      }
    }


    // Check IEC104 parameters
    // Create an object connection from the import file
    if( _unConfigIEC104_getConnectionSettingsFromImportFile(dsFrontEndLine, mIecConnectionFromFile, exceptionInfo) == FALSE )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error creating a connection object from the import file line, due to: " + exceptionInfo[2], "");
      return;
    }

    // Check import file data is consistent
    if( _unConfigIEC104_checkConnectionData(mIecConnectionFromFile, exceptionInfo) == FALSE )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error getting checking data read from file in a connection object, due to: " + exceptionInfo[2], "");
      return;
    }


    // If front end already exists
    if( (dpExists(sFrontEndDp))                      &&
        (dynContains(dsDeleteDps, sFrontEndDp) <= 0)  )
    {
      sFrontEndAlias = unGenericDpFunctions_getAlias(sFrontEndDp);
      iRes           = dpGet(sFrontEndDp  + ".communication.driver_num", iDriverNumber);
      if( iRes >= 0 )
      {
        deError = getLastError();
        if( dynlen(deError) > 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error getting driver number from existant front-end: " + sFrontEndDp + " due to: " + getErrorText(deError), "");
          return;
        }


        // If driver number from current importation is different from the existant one -> Error
        if( iDriverNumber != dsLineAdditionalData[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM] )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> " + getCatStr("unGeneration", "BADEXISTINGPLCDRIVER") + iDriverNumber, "");
          return;
        }


        // Check if IEC connection in the selected driver exits
        if( dpExists("_Iec_" + iDriverNumber + ".Connections.All") == FALSE )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error front end DP exist: " + sFrontEndDp + ", but not IEC104 connection exists: " + "_Iec_" + iDriverNumber, "");
          return;
        }
        else
        {
          iRes = dpGet("_Iec_" + iDriverNumber + ".Connections.All", dsFrontEndIecConnections);
          if( iRes >= 0 )
          {
            deError = getLastError();
            if( dynlen(deError) > 0 )
            {
              fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error getting IEC104 connections for front-end: " + sFrontEndDp + " due to: " + getErrorText(deError), "");
              return;
            }
          }
        }


        // If alias from current device is different from existant one -> Error
        if( sFrontEndAlias != sFrontEndName )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> " + getCatStr("unGeneration", "BADEXISTINGPLCHOSTNAME"), "");
          return;
        }


        // Check IEC104 connection name is the same
        sFrontEndIecConnection = "_" + c_unSystemIntegrity_IEC104 + sFrontEndName;
        if( dynContains(dsFrontEndIecConnections, sFrontEndIecConnection) > 0 )
        {
          // Get current connection data
          if( _unConfigIEC104_getConnectionSettings(sFrontEndIecConnection, mIecConnection, exceptionInfo) == FALSE )
          {
            fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error getting connection parameters for FrontEnd: " + sFrontEndName + " in DP: " + sFrontEndIecConnection + " due to: " + exceptionInfo[2], "");
            return;
          }
          // Check DP data is consistent
          if( _unConfigIEC104_checkConnectionData(mIecConnection, exceptionInfo) == FALSE )
          {
            fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error getting checking data read from the DPEs in:  " + sFrontEndIecConnection + " due to: " + exceptionInfo[2], "");
            return;
          }

          // Check data is similar: DP and impor file
          if( mIecConnection != mIecConnectionFromFile )
          {
            fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> current configuration is different from the existing one, please remove current front-end before importing it", "");
            return;
          }
        }
        else
        {
          // Nothing to do
          // Connection doesn exist, and object connection has been created from the import file and checked already
        }
      }
      else
      {
        fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication() -> Error getting driver number from existent FrontEnd DP: " + sFrontEndDp, "" );
        return;
      }
    }
    else
    {
      // Front-end doesn't exist and connection should not exists:
      // Nothing to do
      // Front End and connection doesn't exist, and object connection has been created from the impor file and checked already
    }
  }
  else
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication: invalid number of arguments, expected " + (iIEC104_IMPORT_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH) + ", and currently are " + iLen , "");
    return;
  }

}// _unConfigIEC104_checkApplication()





/**
Purpose: check data for a sub application (not comm)

Parameters:
    * dsFrontEndLine: dyn_string, input, IEC-104 configuration line in file + additional parameters. It must contains:
        01. Front-end type
        02. Front-end name
        03. Front-end application
        04. IEC Connection - Local list
        05. IEC Connection - Global list
        06. IEC Connection - Active
        07. IEC Connection - Encryption
        08. IEC Connection - Change Role
        09. IEC Connection - Set invalid bit
        10. IEC Connection - Force host connection (0 = Auto, 1 = 1st host, 2 = 2nd host, ...)
        11. IEC Connections: Host IP address A | Host IP port A | Timeout [seconds] A, IP address B | Host IP port B | Timeout [seconds], ...

        (additional parameters)
        01. Driver number
        02. Boolean archive name
        03. Analog archive name
        04. Event16 bits
        05. Event archive name

    * exceptionInfo: dyn_string, output, for errors

Return: nothing
*/
void _unConfigIEC104_setApplication(dyn_string dsFrontEndLine, dyn_string &exceptionInfo)
{
  int iLen, iLoop, iPos;
  string sIecDriverDp, sIecDriverRedundantDp, sIecConnectionDp, sIecConnectionRedundantDp, sFrontEndDp, sIecConnectionName;
  mapping mIecConnection;
  dyn_string dsLine, dsLineAdditional, dsDpToCreate, dsTypeToCreate, dsIecConnections, dsSubApplications, dsImportTimes, dsBoolArchives, dsAnalogArchives, dsEventArchives;
  dyn_errClass deError;


  // Check number of elements
  iLen = dynlen(dsFrontEndLine);
  if( iLen != (iIEC104_IMPORT_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH) )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> invalid number of fields: " + iLen + ", and it is expected: " +  (iIEC104_IMPORT_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH), "");
    return;
  }


  // Separate config line fields and additional parameters fields
  dsLine = dsFrontEndLine;
  for( iLoop = 1 ; iLoop <= UN_CONFIG_PLC_ADDITIONAL_LENGTH ; iLoop++ )
  {
    dynAppend(dsLineAdditional, dsLine[iIEC104_IMPORT_LENGTH + 1]);
    dynRemove(dsLine,                  iIEC104_IMPORT_LENGTH + 1);
  }


  // Create required DPs for the IEC104 driver, IEC104 connection, IEC104 front end and IEC104 system integrity
  sIecDriverDp              = "_Iec_" + dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM];
  sIecDriverRedundantDp     = "_Iec_" + dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]+ "_2";
  sIecConnectionDp          = "_IEC_" + dsLine[UN_CONFIG_PLC_NAME];
  sIecConnectionName        = "IEC_"  + dsLine[UN_CONFIG_PLC_NAME];
  sIecConnectionRedundantDp = "_IEC_" + dsLine[UN_CONFIG_PLC_NAME] + "_2";
  sFrontEndDp               = c_unSystemIntegrity_IEC104 + dsLine[UN_CONFIG_PLC_NAME];

  dynAppend(dsDpToCreate,   sIecDriverDp);
  dynAppend(dsTypeToCreate, "_Iec");

  dynAppend(dsDpToCreate,   sIecDriverRedundantDp);
  dynAppend(dsTypeToCreate, "_Iec");

  dynAppend(dsDpToCreate,   sIecConnectionDp);
  dynAppend(dsTypeToCreate, "_IecConnection");

  dynAppend(dsDpToCreate,   sIecConnectionRedundantDp);
  dynAppend(dsTypeToCreate, "_IecConnection");

  dynAppend(dsDpToCreate,   sFrontEndDp);
  dynAppend(dsTypeToCreate, "IEC104");


  if( _unConfigIEC104_createDps(dsDpToCreate, dsTypeToCreate, "Error creating DPs for the IEC104 driver, connection, Front End and system integrity: ", exceptionInfo) == FALSE )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> error creating front end DPs: " + exceptionInfo[2], "");
    return;
  }


  // Set alias in the frontend
  unConfigGenericFunctions_setAlias(sFrontEndDp, dsLine[UN_CONFIG_PLC_NAME], exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error setting the alias in the IEC104 front end, due to: " + exceptionInfo[2], "");
    return;
  }


  // Update front end DPEs
  if( dpGet(sFrontEndDp + ".configuration.subApplications", dsSubApplications,
            sFrontEndDp + ".configuration.importTime",      dsImportTimes,
            sFrontEndDp + ".configuration.archive_bool",    dsBoolArchives,
            sFrontEndDp + ".configuration.archive_analog",  dsAnalogArchives,
            sFrontEndDp + ".configuration.archive_event",   dsEventArchives) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error getting front end UNICOS configuration", "");
    return;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error getting front end UNICOS configuration, due to: " + exceptionInfo[2], "");
      return;
    }
  }

    // Update old field or new one
  iPos = dynContains(dsSubApplications, dsLine[UN_CONFIG_PLC_SUBAPPLICATION]);
  if( iPos == 0 )
  {
    iPos = dynlen(dsSubApplications) + 1;
  }

    // If the event archive is empty, set it to bool archive
  if (dynlen(dsEventArchives) == 0 )
  {
    dsEventArchives = dsBoolArchives;
  }

    // Populate fields
  dsSubApplications[iPos] = dsLine[UN_CONFIG_PLC_SUBAPPLICATION];
  dsImportTimes    [iPos] = getCurrentTime();
  dsBoolArchives   [iPos] = dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL];
  dsAnalogArchives [iPos] = dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG];
  dsEventArchives  [iPos] = dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT];

  if( dpSetWait(sFrontEndDp + ".configuration.archive_bool",    dsBoolArchives,
                sFrontEndDp + ".configuration.archive_analog",  dsAnalogArchives,
                sFrontEndDp + ".configuration.archive_event",   dsEventArchives,
                sFrontEndDp + ".configuration.subApplications", dsSubApplications,
                sFrontEndDp + ".configuration.importTime",      dsImportTimes,
                sFrontEndDp + ".communication.driver_num",      dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error setting front end UNICOS configuration", "");
    return;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error setting front end UNICOS configuration, due to: " + getErrorText(deError), "");
      return;
    }
  }


  // Get IEC104 connection setup from the import file
  if( _unConfigIEC104_getConnectionSettingsFromImportFile(dsLine, mIecConnection, exceptionInfo) == FALSE )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> error reading IEC104 connection settings from import file, due to: " + exceptionInfo[2], "");
    return;
  }


  // Configure the IEC104 connection
  if( _unConfigIEC104_setConnectionSettings(sIecConnectionDp, mIecConnection, exceptionInfo) == FALSE )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> error setting the IEC104 connection: " + sIecConnectionDp + ", due to: " + exceptionInfo[2], "");
    return;
  }


  // Register IEC104 connection into the driver DP and the driver number in the FrontEnd DP
  if( dpGet(sIecDriverDp + ".Connections.All", dsIecConnections) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error getting IEC104 connections for driver: " + sIecDriverDp, "");
    return;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error getting IEC104 connections for driver: " + sIecDriverDp + ", due to: " + getErrorText(deError), "");
      return;
    }

    if( dynContains(dsIecConnections, sIecConnectionName) == 0 )
    {
      dynAppend(dsIecConnections, sIecConnectionName);
    }
  }


  if( dpSet(sIecDriverDp + ".Connections.All", dsIecConnections,
            sIecDriverDp + ".Connections.Add", sIecConnectionName) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error getting IEC104 connections for driver: " + sIecDriverDp, "");
    return;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error getting IEC104 connections for driver: " + sIecDriverDp + ", due to: " + getErrorText(deError), "");
      return;
    }
  }


  // Create system integrity alarm
  unSystemIntegrity_createSystemAlarm(sFrontEndDp,
                                      DS_pattern,
                                      "Communication with " + dsLine[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsLineAdditional[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
                                      exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error creating system integrity for front end: " + sFrontEndDp + ", due to: " + exceptionInfo[2], "");
    return;
  }


  // Enable system integrity alarm
  if( dpSetWait(UN_SYSTEM_INTEGRITY_IEC104 + UN_SYSTEMINTEGRITY_EXTENSION + ".interface.command",    UN_SYSTEMINTEGRITY_ADD,
                UN_SYSTEM_INTEGRITY_IEC104 + UN_SYSTEMINTEGRITY_EXTENSION + ".interface.parameters", makeDynString(sFrontEndDp)) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error sending SYSTEM INTEGRITY ADD COMMAND for the front end: " + sFrontEndDp, "");
    return;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error sending SYSTEM INTEGRITY ADD COMMAND for the front end: " + sFrontEndDp + ", due to: " + getErrorText(deError), "");
      return;
    }
  }


  // Add system integrity into the application
  _unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern + DS_pattern + c_unSystemIntegrity_IEC104 + dsLine[UN_CONFIG_PLC_NAME],
                                                TRUE,
                                                exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setApplication() -> Error adding system integrity: " + c_unSystemIntegrity_IEC104 + dsLine[UN_CONFIG_PLC_NAME] + " to the Application alert screen, due to: " + exceptionInfo[2], "");
    return;
  }

}//  _unConfigIEC104_setApplication()





void _unConfigIEC104_checkAdditionalData(dyn_string dsLineExtra, dyn_string &exceptionInfo)
{
  int iLen;


  // Checking lenght of archive fields
  iLen = dynlen(dsLineExtra);
  if ( iLen != UN_CONFIG_PLC_ADDITIONAL_LENGTH )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkAdditionalData() -> Invalid number of archive elements, currently " + iLen  + ", and are expected " + UN_CONFIG_PLC_ADDITIONAL_LENGTH, "");
    return;
  }
  else
  {
    // Checking archive elements
    if( dsLineExtra[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL] == "" )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkAdditionalData: bool archive is empty", "");
      return;
    }

    if( dsLineExtra[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG] == "" )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkAdditionalData: analog archive is empty", "");
      return;
    }

    if( dsLineExtra[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT] == "" )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkAdditionalData: event archive is empty", "");
      return;
    }
  }

}// _unConfigIEC104_checkAdditionalData()





void _unConfigIEC104_checkData(dyn_string dsLine, dyn_string &exceptionInfo)
{
  int iLen;
  string sFrontEndName, sApplicationName;


  // Check length of arguments
  iLen = dynlen(dsLine);
  if( iLen != iIEC104_IMPORT_LENGTH )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigOEC104_checkData() -> Invalid number of front end elements, currently " + iLen + ", and are expected " + iIEC104_IMPORT_LENGTH, "");
    return;
  }

  // Check front end type
  unConfigGenericFunctions_checkPLCType(dsLine[UN_CONFIG_PLC_TYPE], exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkData() -> Error checking IEC104 front end definition, due to: " + exceptionInfo[2], "");
    return;
  }
  else
  {
    // Check front end name is not empty
    sFrontEndName = strrtrim(dsLine[UN_CONFIG_PLC_NAME]);
    if( sFrontEndName == "" )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkData() -> empty front end name", "");
      return;
    }

    if( sFrontEndName != dsLine[UN_CONFIG_PLC_NAME] )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkData() -> the front end name contains spaces at the end of the field, please remove them", "");
      return;
    }

    if( unConfigGenericFunctions_nameCheck(sFrontEndName) == FALSE)
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkData: " + sFrontEndName + getCatStr("unGeneration", "BADPLCNAME"), "");
      return;
    }
  }


  // Check application name
  sApplicationName = strrtrim(dsLine[UN_CONFIG_PLC_SUBAPPLICATION]);
  if( sApplicationName == "" )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkData() -> empty application name", "");
    return;
  }
  else
  {
    if( sApplicationName != dsLine[UN_CONFIG_PLC_SUBAPPLICATION] )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkData() -> the front end application name contains spaces at the end of the field, please remove them", "");
      return;
    }

    if( unConfigGenericFunctions_nameCheck(sApplicationName) == FALSE )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkApplication: " + sApplicationName + getCatStr("unGeneration", "BADPLCAPPLICATIONNAME"), "");
      return;
    }
  }

}// _unConfigOEC104_checkData()





/*
Purpose: reads IEC104 front end configuration and creates a IEC104 connection mapping

Parameters:
    * dsFELine: dyn_string, input, array with the front end configuration line in  the import file

    * mIecConnection: mapping, input, mapping with the connection settings

    * exceptionInfo: details of the failure

Return: boolean value: TRUE = operation success, FALSE = operation failed

*/
bool _unConfigIEC104_getConnectionSettingsFromImportFile( dyn_string dsFELine, mapping &mIecConnection, dyn_string &exceptionInfo )
{
  int iLen, iLoop;
  string sConnections;
  dyn_int diConn_Ports, diConn_TimeOut, diConn_Active;
  dyn_string dsConnections, dsLine, dsConn_Hosts, dsLocal, dsGlobal;


  mIecConnection[sIEC104_OBJ_CONN_ACTIVE]       = (int) ((bool) dsFELine[iIEC104_IMPORT_ACTIVE]);
  mIecConnection[sIEC104_OBJ_CONN_ENCRYPTION]   = (int) ((bool) dsFELine[iIEC104_IMPORT_ENCRYPTION]);
  mIecConnection[sIEC104_OBJ_CONN_CHANGE_ROLE]  = (int) ((bool) dsFELine[iIEC104_IMPORT_CHANGE_ROLE]);
  mIecConnection[sIEC104_OBJ_CONN_INVALID_BIT]  = (int) ((bool) dsFELine[iIEC104_IMPORT_INVALID_BIT]);
  mIecConnection[sIEC104_OBJ_CONN_FORCE_ACTIVE] = (int) ((bool) dsFELine[iIEC104_IMPORT_FORCE_ACTIVE]);

  sConnections                                  = dsFELine[iIEC104_IMPORT_CONNECTIONS];
  dsLocal                                       = strsplit(dsFELine[iIEC104_IMPORT_LOCAL_LIST], "|");
  mIecConnection[sIEC104_OBJ_CONN_LOCAL_LIST]   = dsLocal;

  dsGlobal                                      = strsplit(dsFELine[iIEC104_IMPORT_GLOBAL_LIST], "|");
  mIecConnection[sIEC104_OBJ_CONN_GLOBAL_LIST]  = dsGlobal;


  dsConnections = strsplit(sConnections, ",");
  iLen          = dynlen(dsConnections);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    dsLine = strsplit(dsConnections[iLoop], "|");

    if( dynlen(dsLine) == 3 )
    {
      dynAppend(dsConn_Hosts,   dsLine[1]);
      dynAppend(diConn_Ports,   (int) dsLine[2]);
      dynAppend(diConn_TimeOut, (int) dsLine[3]);
      dynAppend(diConn_Active,  1);
    }
    else
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_getConnectionSettingsFromImportFile() -> Error connection entry should have 3 fields separated by | and it is not the case: " + dsLine, "");
      return FALSE;
    }
  }

  mIecConnection[sIEC104_OBJ_CONN_CONN_HOSTS]    = dsConn_Hosts;
  mIecConnection[sIEC104_OBJ_CONN_CONN_PORTS]    = diConn_Ports;
  mIecConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT] = diConn_TimeOut;
  mIecConnection[sIEC104_OBJ_CONN_CONN_ACTIVE]   = diConn_Active;

  return TRUE;
}





/*
Purpose: reads IEC104 connection DP and store its content on an output mapping

Parameters:
    * sIecConnectionDp: string, input, [_IecConnection] DP with the FrontEnd connection configuration

    * mIecConnection: mapping, output, mapping with the connection settings

    * exceptionInfo: details of the failure


Return: boolean value: TRUE = operation success, FALSE = operation failed

*/
bool _unConfigIEC104_getConnectionSettings( string sIecConnectionDp, mapping &mIecConnection, dyn_string &exceptionInfo )
{
  bit32 b32Flags;
  bool bActive;
  int iRes, iLoop, iLen, iForceActive;
  dyn_errClass deError;
  dyn_string dsConnectionInfo, dsLocal, dsGlobal;


  // Check connection DP exists
  if( dpExists(sIecConnectionDp) == FALSE )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_getConnectionSettings() -> Error, IEC104 connection DP doesn't exist: " + sIecConnectionDp, "");
    return FALSE;
  }


  // Get connection data
  iRes = dpGet( sIecConnectionDp + ".LocalList",            mIecConnection[sIEC104_OBJ_CONN_LOCAL_LIST],
                sIecConnectionDp + ".GlobalList",           mIecConnection[sIEC104_OBJ_CONN_GLOBAL_LIST],
                sIecConnectionDp + ".Config.ConnInfo",      dsConnectionInfo,
                sIecConnectionDp + ".Config.Active",        bActive,
                sIecConnectionDp + ".Config.Flags",         b32Flags,
                sIecConnectionDp + ".Config.ForceActive",   iForceActive,
                sIecConnectionDp + ".Config.SetInvalidBit", mIecConnection[sIEC104_OBJ_CONN_INVALID_BIT]);
  if( iRes != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_getConnectionSettings() -> Error getting DPE contents of connection: " + sIecConnectionDp, "");
    return FALSE;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_getConnectionSettings() -> Error getting DPE contents of connection: " + sIecConnectionDp + ", due to: " + exceptionInfo[2], "");
      return FALSE;
    }
  }
  mIecConnection[sIEC104_OBJ_CONN_ACTIVE]       = (int) bActive;
  mIecConnection[sIEC104_OBJ_CONN_FORCE_ACTIVE] = iForceActive;


  // Poppulate IEC104 connection object
  mIecConnection[sIEC104_OBJ_CONN_ENCRYPTION] = getBit(b32Flags, 0);
  if( mIecConnection[sIEC104_OBJ_CONN_ENCRYPTION] == -1 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_getConnectionSettings() -> Error getting IEC104 connection encryption value from the DPE", "");
    return FALSE;
  }

  mIecConnection[sIEC104_OBJ_CONN_CHANGE_ROLE] = getBit(b32Flags, 8);
  if( mIecConnection[sIEC104_OBJ_CONN_CHANGE_ROLE] == -1 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_getConnectionSettings() -> Error getting IEC104 connection change role setting value from the DPE", "");
    return FALSE;
  }


  iLen = dynlen(dsConnectionInfo);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    string sLine;
    dyn_string dsTemp;

    sLine = dsConnectionInfo[iLoop];
    dsTemp = strsplit(sLine, " ");

    if( dsTemp[1] == "104" )
    {
      dynAppend(mIecConnection[sIEC104_OBJ_CONN_CONN_HOSTS],    dsTemp[2]);
      dynAppend(mIecConnection[sIEC104_OBJ_CONN_CONN_PORTS],    (int) dsTemp[3]);
      dynAppend(mIecConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT], (int) dsTemp[4]);
      dynAppend(mIecConnection[sIEC104_OBJ_CONN_CONN_ACTIVE],   (int) dsTemp[5]);
    }

  }

  return TRUE;
}





/*
Purpose: set a IEC104 connection DP and store its content from a mapping object

Parameters:
    * sIecConnectionDp: string, input, [_IecConnection] DP with the FrontEnd connection configuration

    * mIecConnection: mapping, input, mapping with the connection settings

    * exceptionInfo: details of the failure


Return: boolean value: TRUE = operation success, FALSE = operation failed

*/
bool _unConfigIEC104_setConnectionSettings( string sIecConnectionDp, mapping mIecConnection, dyn_string &exceptionInfo )
{
  bit32 b32Flags;
  int iLoop, iLen;
  dyn_string dsMappingKeys, dsMissingKeys, dsMissingValue, dsConnections;
  dyn_errClass deError;


  // Create connection DP
  if( _unConfigIEC104_createDps(makeDynString(sIecConnectionDp),
                                makeDynString("_IecConnection"),
                                "Error creating IEC104 connection settings: " + sIecConnectionDp,
                                exceptionInfo                                                    ) == FALSE )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error creating IEC connection DP: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Checking mapping has the enough fields
  dsMappingKeys = makeDynString(sIEC104_OBJ_CONN_LOCAL_LIST,
                                sIEC104_OBJ_CONN_GLOBAL_LIST,
                                sIEC104_OBJ_CONN_ACTIVE,
                                sIEC104_OBJ_CONN_ENCRYPTION,
                                sIEC104_OBJ_CONN_CHANGE_ROLE,
                                sIEC104_OBJ_CONN_INVALID_BIT,
                                sIEC104_OBJ_CONN_FORCE_ACTIVE,
                                sIEC104_OBJ_CONN_CONN_HOSTS,
                                sIEC104_OBJ_CONN_CONN_PORTS,
                                sIEC104_OBJ_CONN_CONN_TIME_OUT,
                                sIEC104_OBJ_CONN_CONN_ACTIVE);

  iLen = dynlen(dsMappingKeys);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( mappingHasKey(mIecConnection, dsMappingKeys[iLoop]) == FALSE )
    {
      dynAppend(dsMissingKeys, dsMappingKeys[iLoop]);
      continue;
    }

    if( mIecConnection[dsMappingKeys[iLoop]] == "" )
    {
      dynAppend(dsMissingValue, dsMappingKeys[iLoop]);
      continue;
    }
  }

  if( (dynlen(dsMissingKeys)  > 0) &&
      (dynlen(dsMissingValue) > 0)  )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error, input mapping has not following fields: " + dsMissingKeys +  " or following fields are empty: " + dsMissingValues, "");
    return FALSE;
  }


  // Creating connection fields
  iLen = dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_HOSTS]);
  if( !((iLen                                                   == dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_PORTS]))    &&
        (dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_PORTS])    == dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT])) &&
        (dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT]) == dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_ACTIVE]))     ))
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error, input mapping has different size on fields: sIEC104_OBJ_CONN_CONN_HOSTS, sIEC104_OBJ_CONN_CONN_PORTS, sIEC104_OBJ_CONN_CONN_TIME_OUT, sIEC104_OBJ_CONN_CONN_ACTIVE", "");
    return FALSE;
  }

  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    dynAppend(dsConnections, "104"                                                 + " " +
                             mIecConnection[sIEC104_OBJ_CONN_CONN_HOSTS]   [iLoop] + " " +
                             mIecConnection[sIEC104_OBJ_CONN_CONN_PORTS]   [iLoop] + " " +
                             mIecConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT][iLoop] + " " +
                             mIecConnection[sIEC104_OBJ_CONN_CONN_ACTIVE]  [iLoop]);
  }


  // Calculate flag field
  if( setBit(b32Flags, 0, mIecConnection[sIEC104_OBJ_CONN_ENCRYPTION]) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error setting IEC104 connection encryption value", "");
    return FALSE;
  }

  if( setBit(b32Flags, 8, mIecConnection[sIEC104_OBJ_CONN_CHANGE_ROLE]) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error setting IEC104 connection change role value", "");
    return FALSE;
  }


  // Set DPE value in DP connection
  if( dpSet(sIecConnectionDp + ".LocalList",            mIecConnection[sIEC104_OBJ_CONN_LOCAL_LIST],
            sIecConnectionDp + ".GlobalList",           mIecConnection[sIEC104_OBJ_CONN_GLOBAL_LIST],
            sIecConnectionDp + ".Config.ConnInfo",      dsConnections,
            sIecConnectionDp + ".Config.Active",        mIecConnection[sIEC104_OBJ_CONN_ACTIVE],
            sIecConnectionDp + ".Config.Flags",         b32Flags,
            sIecConnectionDp + ".Config.ForceActive",   mIecConnection[sIEC104_OBJ_CONN_FORCE_ACTIVE],
            sIecConnectionDp + ".Config.SetInvalidBit", mIecConnection[sIEC104_OBJ_CONN_INVALID_BIT]) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error setting DPE values to the IEC connection DP: " + sIecConnectionDp, "");
    return FALSE;
  }
  else
  {
    deError = getLastError();
    if( dynlen(deError) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_setConnectionSettings() -> Error setting DPE values to the IEC connection DP: " + sIecConnectionDp + " due to: " + getErrorText(deError), "");
      return FALSE;
    }
  }

  return TRUE;
}





/*
Purpose: check a IEC104 mapping connection object parameters

Parameters:

    * mIecConnection: mapping, input, mapping with the connection settings

    * exceptionInfo: details of the failure


Return: boolean value: TRUE = operation success, FALSE = operation failed

*/
bool _unConfigIEC104_checkConnectionData(mapping mConnection, dyn_string &exceptionInfo)
{
  int iLen;
  dyn_int diPort, diTimeOut, diActive;
  dyn_string dsHost;


  // Check mapping has all required fields
  _checkMappingKeys(mConnection,
                    makeDynString(sIEC104_OBJ_CONN_LOCAL_LIST,
                                  sIEC104_OBJ_CONN_GLOBAL_LIST,
                                  sIEC104_OBJ_CONN_ACTIVE,
                                  sIEC104_OBJ_CONN_ENCRYPTION,
                                  sIEC104_OBJ_CONN_CHANGE_ROLE,
                                  sIEC104_OBJ_CONN_INVALID_BIT,
                                  sIEC104_OBJ_CONN_FORCE_ACTIVE,
                                  sIEC104_OBJ_CONN_CONN_HOSTS,
                                  sIEC104_OBJ_CONN_CONN_PORTS,
                                  sIEC104_OBJ_CONN_CONN_TIME_OUT,
                                  sIEC104_OBJ_CONN_CONN_ACTIVE),
                    exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error checking the input mapping with the IEC connection settings, due to: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Checking mapping global and local lists values
  _checkMappingList(mConnection,
                    makeDynString(sIEC104_OBJ_CONN_LOCAL_LIST,
                                  sIEC104_OBJ_CONN_GLOBAL_LIST),
                    exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connections list, due to: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Checking mapping value types
  _checkMappingType(mConnection,
                    makeDynString(sIEC104_OBJ_CONN_ACTIVE,
                                  sIEC104_OBJ_CONN_ENCRYPTION,
                                  sIEC104_OBJ_CONN_CHANGE_ROLE,
                                  sIEC104_OBJ_CONN_FORCE_ACTIVE,
                                  sIEC104_OBJ_CONN_CONN_HOSTS,
                                  sIEC104_OBJ_CONN_CONN_PORTS,
                                  sIEC104_OBJ_CONN_CONN_TIME_OUT,
                                  sIEC104_OBJ_CONN_CONN_ACTIVE),
                    makeDynString(INT_VAR,
                                  INT_VAR,
                                  INT_VAR,
                                  INT_VAR,
                                  DYN_STRING_VAR,
                                  DYN_INT_VAR,
                                  DYN_INT_VAR,
                                  DYN_INT_VAR),
                    exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connection booleans, due to: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Checking Force active value is not negative
  if( mConnection[sIEC104_OBJ_CONN_FORCE_ACTIVE] < 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, Force Active value can't be negative: " + mConnection[sIEC104_OBJ_CONN_FORCE_ACTIVE], "");
    return FALSE;
  }


   // Checking mapping invalid bit
  _checkMappingInvalidBit(mConnection, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connection invalid bit setting, due to: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Checking connection information: hosts, ports, timeout, active
    // Check arrays have same lenght
  dsHost    = mConnection[sIEC104_OBJ_CONN_CONN_HOSTS];
  diPort    = mConnection[sIEC104_OBJ_CONN_CONN_PORTS];
  diTimeOut = mConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT];
  diActive  = mConnection[sIEC104_OBJ_CONN_CONN_ACTIVE];
  if( !((dynlen(dsHost)    == dynlen(diPort))    &&
        (dynlen(diPort)    == dynlen(diTimeOut)) &&
        (dynlen(diTimeOut) == dynlen(diActive)) ))
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connection, different number of items in hosts, ports, timeouts and activations", "");
    return FALSE;
  }

    // Check arrays of hosts, ports, timeouts and activations are dyn_string and are not empty
  _checkMappingArraysEmpty(mConnection,
                           makeDynString(sIEC104_OBJ_CONN_CONN_HOSTS,
                                         sIEC104_OBJ_CONN_CONN_PORTS,
                                         sIEC104_OBJ_CONN_CONN_TIME_OUT,
                                         sIEC104_OBJ_CONN_CONN_ACTIVE),
                           exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking if any dyn_string array has a field empty : " + exceptionInfo[2], "");
    return FALSE;
  }


  // Check array of ports is bigger than 0 and < 65536
  _checkMappingPorts(mConnection[sIEC104_OBJ_CONN_CONN_PORTS], exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connection ports are inside range: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Check array of timeouts are biggers than 0
  _checkMappingTimeOut(mConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT], exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connection time outs are inside range: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Check array of activations are 0 or 1
  _checkMappingActivation(mConnection[sIEC104_OBJ_CONN_CONN_ACTIVE], exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_checkConnectionData() -> Error, checking connection activation are inside range: " + exceptionInfo[2], "");
    return FALSE;
  }

  return TRUE;
}





void _checkMappingActivation(dyn_int diActive, dyn_string &exceptionInfo)
{
  int iLoop, iLen;


  iLen = dynlen(diActive);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    switch( diActive[iLoop] )
    {
      case 0:
      case 1:
        break;

      default:
        fwException_raise(exceptionInfo, "ERROR", "_checkMappingActivation() -> Error, connection active is: " + diActive[iLoop] + " and only can be: 0 or 1", "");
        return;
        break;
    }
  }

} // _checkMappingTimeOut()





void _checkMappingTimeOut(dyn_int diTimeOut, dyn_string &exceptionInfo)
{
  int iLoop, iLen;


  iLen = dynlen(diTimeOut);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( diTimeOut[iLoop] < 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_checkMappingTimeOut() -> Error in time out: " + diTimeOut[iLoop] + ", it should be 0 or greater", "");
      return;
    }
  }

} // _checkMappingTimeOut()





void _checkMappingPorts(dyn_int diPorts, dyn_string &exceptionInfo)
{
  int iLoop, iLen;


  iLen = dynlen(diPorts);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( (diPorts[iLoop] < 0 )   ||
        (diPorts[iLoop] > 65535)  )
    {
      fwException_raise(exceptionInfo, "ERROR", "_checkMappingPorts() -> Error in port number: " + diPorts[iLoop] + ", it should be in the range [0 - 65535]", "");
      return;
    }
  }

} // _checkMappingPorts()





void _checkMappingInvalidBit(mapping mConnection, dyn_string &exceptionInfo)
{
  int iValue;


  unConfigGenericFunctions_checkInt(mConnection[sIEC104_OBJ_CONN_INVALID_BIT], iValue, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    exceptionInfo[2] = " -> _checkInvalidBit() -> Error checking if connection invalid bit is an integer, due to: " + exceptionInfo[2];
    return;
  }

  switch( iValue )
  {
    case 0:  // Do not set invalid bits
    case 1:  // Set invalid bit with current timestamp
    case 2:  // set invalid bit without changing the source timestamp
      break;

    default:
      fwException_raise(exceptionInfo, "ERROR", "_checkInvalidBit() -> Error, invalid bit value is out of range (0-2): " +  iValue, "");
      return;
      break;
  }

}





// This function will check dyn_string with strings like
//
// ("*.*.*.*.*", "1.*.*.*.*", "2.*.*.*.*")
//
void _checkMappingList(mapping mConnections, dyn_string dsKeys, dyn_string &exceptionInfo)
{
  int iLen, iLoop, iLenList, iLoopList, iLenDots, iLoopDots;
  string sList;
  dyn_string dsList, dsNumericField;


  iLen = dynlen(dsKeys);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    // Checking each selected key from the mapping
    dsList   = mConnections[dsKeys[iLoop]];
    iLenList = dynlen(dsList);

    for( iLoopList = 1 ; iLoopList <= iLenList ; iLoopList++ )
    {
      // Checking all entries from the list
      sList    = dsList[iLoopList];

      // Extract number of dots
      dsNumericField = strsplit(sList, ".");
      iLenDots       = dynlen(dsNumericField);
      if( iLenDots != 5 )
      {
        fwException_raise(exceptionInfo, "ERROR", "_checkMappingList() -> Error in connection setting " + dsKeys[iLoop] + ": " + sList + " . Invalid number of fields. It should have 5 integers / asterisks  separated by dots, like: '1.*.*.*.*'", "");
        return;
      }

      // Checking each numeric field (integer 0-255 or asterisk)
      for( iLoopDots = 1 ; iLoopDots <= iLenDots ; iLoopDots++ )
      {
        if( dsNumericField[iLoopDots] != "*" )
        {
          int iValue;
          unConfigGenericFunctions_checkInt(dsNumericField[iLoopDots], iValue, exceptionInfo);
          if( dynlen(exceptionInfo) > 0 )
          {
            exceptionInfo[2] = "_checkMappingList() -> Error in connection setting " + dsKeys[iLoop] + ": " + sList + " . Field " + iLoopDots + " is not an integeger or asterisk: " + dsNumericField[iLoopDots] + " -> " + exceptionInfo[2];
            return;
          }

          if( (dsNumericField[iLoopDots] < 0)  ||
              (dsNumericField[iLoopDots] > 255) )
          {
            fwException_raise(exceptionInfo, "ERROR", "_checkMappingList() -> Error in connection setting " + dsKeys[iLoop] + ": " + sList + " . Field " + iLoopDots + " is and integer out of range (0-255): " + dsNumericField[iLoopDots], "");
            return;
          }
        }
      }
    }
  }

}// _checkMappingList()





void _checkMappingType(mapping mConnection, dyn_string dsKeys, dyn_string dsTypes, dyn_string &exceptionInfo)
{
  int iDataType, iLen, iLoop;


  iLen = dynlen(dsKeys);
  for( iLoop =  1 ; iLoop <= iLen ; iLoop++ )
  {
    iDataType = getType(mConnection[dsKeys[iLoop]]);
    if( iDataType == 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_checkMappingType() -> Error getting data type from connection setting: " + dsKeys[iLoop], "");
      return;
    }
    else
    {
      if( iDataType != dsTypes[iLoop] )
      {
        fwException_raise(exceptionInfo, "ERROR", "_checkMappingType() -> Error in connection setting: " + dsKeys[iLoop] + ", it is a type " + getTypeName(mConnection[dsKeys[iLoop]]) + ", and should be " + getTypeName(dsTypes[iLoop]), "");
        return;
      }
    }
  }

}// _checkMappingType()





void _checkMappingInt(mapping mConnection, dyn_string dsKeys, dyn_string &exceptionInfo)
{
  int iDataType, iLen, iLoop;


  iLen = dynlen(dsKeys);
  for( iLoop =  1 ; iLoop <= iLen ; iLoop++ )
  {
    iDataType = getType(mConnection[dsKeys[iLoop]]);
    if( iDataType == 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_checkMappingInt() -> Error getting data type from connection setting: " + dsKeys[iLoop], "");
      return;
    }
    else
    {
      if( iDataType != INT_VAR )
      {
        fwException_raise(exceptionInfo, "ERROR", "_checkMappingInt() -> Error in connection setting: " + dsKeys[iLoop] + ", it is a type " + getTypeName(mConnection[dsKeys[iLoop]]) + ", and should be an integer value", "");
        return;
      }
    }
  }

}// _checkMappingBoolean()





void _checkMappingArraysEmpty(mapping mConnection, dyn_string dsKeys, dyn_string &exceptionInfo)
{
  int iLen, iLoop, iLenArray, iLoopArray;
  dyn_string dsArray;


  iLen = dynlen(dsKeys);
  for( iLoop =  1 ; iLoop <= iLen ; iLoop++ )
  {
    // Check if any array elemenent is empty
    dsArray   = mConnection[dsKeys[iLoop]];
    iLenArray = dynlen(dsArray);
    for( iLoopArray = 1 ; iLoopArray <= iLenArray ; iLoopArray++ )
    {
      if( dsArray[iLoopArray] == "" )
      {
        fwException_raise(exceptionInfo, "ERROR", "_checkMappingArraysEmpty() -> Error in connection setting: " + dsKeys[iLoop] + " array, it has its field number " + iLoopArray + " empty", "");
        return;
      }
    }
  }

}// _checkMappingArraysEmpty()





void _checkMappingKeys(mapping mConnection, dyn_string dsKeys, dyn_string &exceptionInfo)
{
  int iLen, iLoop;
  dyn_string dsMissingKeys;


  iLen = dynlen(dsKeys);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( mappingHasKey(mConnection, dsKeys[iLoop]) == FALSE )
    {
      dynAppend(dsMissingKeys, dsKeys[iLoop]);
    }
  }

  if( dynlen(dsMissingKeys) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "_checkMappingKeys() -> Error missing keys in the mapping: " + dsMissingKeys, "");
    return;
  }
}





/**
Purpose: return the driver config of IEC104 front-end

Parameters:
      iDriverNumber     int,      input,   The driver number
      sName             string,   output,  The name to display
      sDriverName       string,   output,  The manager name
      iManType          int,      output,  The manager type
      sCommandLine      int,      output,  The command line to start the manager
*/
void IEC104_getFrontEndManagerConfig(int iDriverNumber, string &sName, string &sDriverName, int &iManType, string &sCommandLine)
{

  sName        = "IEC104";
  sDriverName  = "WCCOAiec";
  iManType     = DRIVER_MAN;
  sCommandLine = "-num " + iDriverNumber;

}//  IEC104_getFrontEndManagerConfig()





/**
Purpose: return the list of Dps that wil be archived during the set of the OPC front-end

Parameters:
      sCommand       string,     input,    The current entry in the import file
      sDp            string,     input,    The given dp: FrontEnd or SystemAlarm DP
      sFEType        string,     input,    The front-end type
      dsArchiveDp    dyn_string, output,   List of archivedDp

*/
void IEC104_getFrontEndArchiveDp(string sCommand, string sDp, string sFEType, dyn_string &dsArchiveDp)
{

  switch( sCommand )
  {
    case UN_PLC_COMMAND:
      dsArchiveDp = makeDynString();
      break;

    default:
      dsArchiveDp = makeDynString();
      break;
  }

}//  IEC104_getFrontEndArchiveDp()





/**
Purpose: check the coherency of the delete command

Parameters:
  dsLine,           dyn_string,    input,    Config dyn_string
                                             dsLine[1] = DELETE
                                             dsLine[2] = FrontEnd name
                                             dsLine[3] = FrontEnd application name
                                             dsLine[4] = Device type
                                             dsLine[5] = Device number
  dsDeleteDps,      dyn_string,    output,   List of dp that will be deleted
  exceptionInfo,    dyn_string,    output,   Output for errors
*/
void unConfigIEC104_checkDeleteCommand(dyn_string dsLine, dyn_string &dsDeleteDps, dyn_string &exceptionInfo)
{
  int iLen;
  string sFrontEndName, sFrontEndApplication, sObjectType, sDeviceNumber, sFrontEndDp, sFrontEndConnection;
  dyn_string dsAlarmDPs;


  iLen = dynlen(dsLine);
  if( iLen >= 2 )
  {
    // At least, DELETE command and front end name

    // Get front end name
    sFrontEndName = dsLine[2];

    if( (sFrontEndName != "" )   &&
        (dsLine[1]     == UN_DELETE_COMMAND) )
    {
      // Get application
      if( iLen >= 3 )
      {
        sFrontEndApplication = dsLine[3];
      }

      // Get device type
      if( iLen >= 4 )
      {
        sObjectType = dsLine[4];
      }

      // Get device number
      if( iLen >= 5 )
      {
        // Get device number
        sDeviceNumber = dsLine[5];
      }


      unGenericDpFunctions_getPlcDevices("", sFrontEndName, sFrontEndApplication, sObjectType, sDeviceNumber, dsDeleteDps);

      // If there is only the DELETE keyword, then remove also: FrontEnd, FrontEnd connection, and system integrity alarms
      if( ( sFrontEndName        != "" )  &&
          ( sFrontEndApplication != "" )  &&
          ( sObjectType          == "" )  &&
          ( sDeviceNumber        == "")     )
      {
        // sFrontEndDp is IEC104_[sFrontEndName]
        sFrontEndDp = c_unSystemIntegrity_IEC104 + sFrontEndName;
        if( dpExists(sFrontEndDp) )
        {
          dynAppend(dsDeleteDps, sFrontEndDp);
        }

        // sFrontEnd connection
        sFrontEndConnection = "_IEC_" + sFrontEndName;
        if( dpExists(sFrontEndConnection) )
        {
          dynAppend(dsDeleteDps, sFrontEndConnection);
        }

        // Add system integrity
        dsAlarmDPs = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sFrontEndDp, strpos(sFrontEndDp, ":") + 1), c_unSystemAlarm_dpType);
        dynAppend(dsDeleteDps, dsAlarmDPs);
      }
    }
    else
    {
      fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkDeleteCommand() -> Error front end name missing in DELETE statement: " + sFrontEndName + ", or wrong DELETE statement: " + dsLine[1], "");
      return;
    }
  }
  else
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkDeleteCommand() -> Error in DELETE statement, not enough arguments: " + iLen + ", it is required at least 2", "");
    return;
  }

}//  unConfigIEC104_checkDeleteCommand()





/**
Purpose: delete function for the IEC104 front end

Parameters:
    dsLine,          dyn_string,    input,      Delete line in the config file
    exceptionInfo,   dyn_string,    output,     Output error exceptions

Return: nothing
*/
void unConfigIEC104_deleteCommand(dyn_string dsLine, dyn_string &exceptionInfo)
{
  int iLen;
  string sApplication, sObjectType, sDeviceNumber;
  dyn_string dsDpeList, exceptionInfo;


  // Check before removal
  unConfigIEC104_checkDeleteCommand(dsLine, dsDpeList, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_deleteCommand() -> Error getting the list of devices to remove, due to: " + exceptionInfo[2], "");
    return;
  }


  iLen = dynlen(dsLine);
  if( iLen >= 2 )
  {
    // At least DELETE line includes: DELETE and FrontEnd name

    // Application name
    if( iLen >= 3 )
    {
      sApplication = dsLine[3];
    }

    // Device type
    if( iLen >= 4 )
    {
      sObjectType = dsLine[4];
    }

    // Device number
    if( iLen >= 5 )
    {
      sDeviceNumber = dsLine[5];
    }

    // Remove devices associated with this front end
    unGenericDpFunctions_deletePlcDevices(dsLine[2], sApplication, sObjectType, sDeviceNumber, "progressBar", exceptionInfo);
    if( dynlen( exceptionInfo ) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_deleteCommand() -> Error removing devices linked with front end " + dsLine[2] + ", due to: ", exceptionInfo[2], "");
      return;
    }


    if( (dsLine[2]     != "")  &&
//        (sApplication  == "")  &&
        (sObjectType   == "")  &&
        (sDeviceNumber == "")    )
    {
      _unConfigIEC104_deleteFrontEnd(dsLine[2], exceptionInfo);
        if( dynlen( exceptionInfo ) > 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_deleteCommand() -> Error removing front end: " + dsLine[2] + ", due to: ", exceptionInfo[2], "");
        return;
      }
    }
  }

}//  unConfigIEC104_deleteCommand()





void _unConfigIEC104_deleteFrontEnd(string sFrontEndName, dyn_string &exceptionInfo)
{
  int iDriverNumber, iRes, iLen, iLoop;
  string sFrontEndDp, sIec104Connection;
  dyn_string dsAlarmDps;
  dyn_errClass deError;


  sFrontEndDp = c_unSystemIntegrity_IEC104 + sFrontEndName;
  if( dpExists(sFrontEndDp) == FALSE )
  {
    sFrontEndDp = "";
  }


  if( sFrontEndDp != "" )
  {
    if( unGenericDpFunctions_getSystemName(sFrontEndName) == getSystemName() )
    {
      // Get front end driver number
      iRes = dpGet(sFrontEndDp + ".communication.driver_num", iDriverNumber);
      if( iRes != 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_deleteFrontEnd() -> Error getting driver number for front end: " + sFrontEndName, "");
        return;
      }
      else
      {
        deError = getLastError(deError);
        if( dynlen(deError) > 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_deleteFrontEnd() -> Error getting driver number for front end: " + sFrontEndName + ", due to: " + getErrorText(deError), "");
          return;
        }
      }


      // Remove system integrity alarms
      dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + sFrontEndDp, c_unSystemAlarm_dpType);
      iLen       = dynlen(dsAlarmDps);
      for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
      {
        _unSystemIntegrity_modifyApplicationAlertList(dsAlarmDps[iLoop], FALSE, exceptionInfo);
        if( dynlen(exceptionInfo) > 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_deleteFrontEnd() -> Error removing system integrity: " + dsAlarmDps[iLoop] + " from the Application alert screen, due to: " + exceptionInfo[2], "");
          return;
        }
      }


      // Unregister system integrity alarms for the front end
      if( dpSetWait(UN_SYSTEM_INTEGRITY_IEC104 + UN_SYSTEMINTEGRITY_EXTENSION + ".interface.command",    UN_SYSTEMINTEGRITY_DELETE,
                    UN_SYSTEM_INTEGRITY_IEC104 + UN_SYSTEMINTEGRITY_EXTENSION + ".interface.parameters", makeDynString(sFrontEndDp)) != 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_deleteFrontEnd() -> Error sending SYSTEM INTEGRITY DELETE COMMAND for the front end: " + sFrontEndDp, "");
        return;
      }
      else
      {
        deError = getLastError();
        if( dynlen(deError) > 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_deleteFrontEnd() -> Error sending SYSTEM INTEGRITY DELETE COMMAND for the front end: " + sFrontEndDp + ", due to: " + getErrorText(deError), "");
          return;
        }
      }


      // Unregister IEC104 connection from the IEC104 drivers
      sIec104Connection = "_IEC_" + sFrontEndName;
      if( _unConfigIEC104_unregisterConnection(sIec104Connection, exceptionInfo) == FALSE )
      {
        return;
      }


      // Remove IEC104 connection
      if( _unConfigIEC104_removeDps(sIec104Connection, "_unConfigIEC104_deleteFrontEnd() -> Error removing IEC104 connection: ", exceptionInfo) == FALSE )
      {
        return;
      }


      // Remove front end DP
      if( _unConfigIEC104_removeDps(sFrontEndDp, "_unConfigIEC104_deleteFrontEnd() -> Error removing front end DP: ", exceptionInfo) == FALSE )
      {
        return;
      }

    }
    else
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_deleteFrontEnd() -> " + sFrontEndDp + getCatStr("unGeneral", "UNOBJECTNOTDELETEDBADSYSTEM"), "");
      return;
    }
  }

}//  _unConfigIEC104_deleteFrontEnd()





bool _unConfigIEC104_unregisterConnection(string sIec104Connection, dyn_string &exceptionInfo)
{
  bool bUpdate;
  int iLen, iLoop, iPos, iRes;
  string sChar, sConnection;
  dyn_int  diPositions;
  dyn_string dsDrivers;
  dyn_dyn_string ddsConnections;
  dyn_errClass deError;


  sChar       = substr(sIec104Connection, 0, 1);
  sConnection = sIec104Connection;

  // Remove the _ character
  if ( sChar = "_" )
  {
    sConnection = substr(sIec104Connection, 1);
  }


  // Get all connections from all IEC104 drivers
  dsDrivers = dpNames("_Iec_*.Connections.All", "_Iec");
  iLen = dynlen(dsDrivers);
  if( iLen > 0 )
  {
    iRes = dpGet(dsDrivers, ddsConnections);
    if( iRes != 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_unregisterConnection() -> Error getting IEC104 driver connections", "");
      return FALSE;
    }
    else
    {
      deError = getLastError();
      if( dynlen(deError) > 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_unregisterConnection() -> Error getting IEC104 driver connections, due to: " + getErrorText(deError), "");
        return FALSE;
      }
    }
  }


  // Find the connection in the driver and remove it for there.
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    // Check removed connection is not defined
    iPos    = 1;
    bUpdate = FALSE;

    while( iPos > 0 )
    {
      iPos = dynContains(ddsConnections[iLoop], sConnection);
      if( iPos > 0 )
      {
        if( dynRemove(ddsConnections[iLoop], iPos) != 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_unregisterConnection() -> Removing connection from " + dsDrivers[iLoop] + " in position: " + iPos, "");
          return FALSE;
        }
        bUpdate = TRUE;
      }
    }


    if( bUpdate )
    {
      iRes = dpSet(dsDrivers[iLoop], ddsConnections[iLoop]);
      if( iRes != 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_unregisterConnection() -> Error updating IEC104 connections for driver: " + dsDrivers[iLoop], "");
        return FALSE;
      }
      else
      {
        deError = getLastError();
        if( dynlen(deError) > 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_unregisterConnection() -> Error updating IEC104 connections for driver: " + dsDrivers[iLoop] + ", due to: " + getErrorText(deError), "");
          return FALSE;
        }
      }
    }
  }

  return TRUE;

}//   _unConfigIEC104_unregisterConnection()





bool _unConfigIEC104_removeDps(dyn_string dsDp, string sErrorMessage, dyn_string &exceptionInfo)
{
  int iLen, iLoop, iRes;
  dyn_errClass deError;


  iLen = dynlen(dsDp);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( dpExists(dsDp[iLoop]) )
    {
      iRes = dpDelete(dsDp[iLoop]);
      if( iRes != 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", sErrorMessage + ", " + dsDp[iLoop], "");
        return FALSE;
      }
      else
      {
        deError = getLastError();
        if( dynlen(deError) )
        {
          fwException_raise(exceptionInfo, "ERROR", sErrorMessage + ", " + dsDp[iLoop] + ", due to: " + getErrorText(deError), "");
          return FALSE;
        }
      }
    }
  }

  return TRUE;

}//  _unConfigIEC104_removeDps()





bool _unConfigIEC104_createDps(dyn_string dsDp, dyn_string dsType, string sErrorMessage, dyn_string &exceptionInfo)
{
  int iLen, iLoop, iRes;
  dyn_errClass deError;


  iLen = dynlen(dsDp);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( dpExists(dsDp[iLoop]) == FALSE )
    {
      iRes = dpCreate(dsDp[iLoop], dsType[iLoop]);
      if( iRes != 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", sErrorMessage + ", " + dsDp[iLoop] + " of type: " + dsType[iLoop], "");
        return FALSE;
      }
      else
      {
        deError = getLastError();
        if( dynlen(deError) )
        {
          fwException_raise(exceptionInfo, "ERROR", sErrorMessage + ", " + dsDp[iLoop] + " of type: " + dsType[iLoop] + ", due to: " + getErrorText(deError), "");
          return FALSE;
        }
      }
    }
  }

  return TRUE;

}//  _unConfigIEC104_createDps()





void IEC104_Com_ExportConfig(dyn_string dsParams, dyn_string &exceptionInfo)
{
  int iLen, iLoop;
  string sIecConnectionDp, sGlobal, sLocal, sHosts;
  mapping mIecConnection;
  dyn_string dsOutput, dsGlobal, dsLocal, dsHosts;


  // Checking input parameters
  iLen = dynlen(dsParams);
  if( iLen == UN_CONFIG_EXPORT_LENGTH )
  {
    // Delete header
    dynAppend(dsOutput, "# Uncomment the line below to delete all the devices and the front-end: " + dsParams[UN_CONFIG_EXPORT_PLCNAME]);
    unExportDevice_writeDsStringToFile(dsOutput, UN_PARAMETER_DELIMITER);

    // Delete statement
    dynClear (dsOutput);
    dynAppend(dsOutput, "# " + UN_DELETE_COMMAND);
    dynAppend(dsOutput, dsParams[UN_CONFIG_EXPORT_PLCNAME]);
    unExportDevice_writeDsStringToFile(dsOutput, UN_PARAMETER_DELIMITER);

    // Empty line
    dynClear (dsOutput);
    dynAppend(dsOutput, "");
    unExportDevice_writeDsStringToFile(dsOutput, "");

    // Front end header
    dynClear (dsOutput);
    dynAppend(dsOutput, "# Config Line : PLCCONFIG;type;fe_name;app_name;EIC_connection_local_list;EIC_connection_global_list;EIC_connection_active;EIC_connection_encryption;EIC_connection_change_role;EIC_connection_invalid_bit;Force_Active_Connection;HostA_IP|HostA_port|TimeoutA_sec,HostB_IP|HostB_port|TimeoutB_sec,...");
    unExportDevice_writeDsStringToFile(dsOutput, UN_PARAMETER_DELIMITER);

    // Front end statement
    dynClear(dsOutput);
    dynAppend(dsOutput, UN_PLC_COMMAND);
    dynAppend(dsOutput, "IEC104");
    dynAppend(dsOutput, dsParams[UN_CONFIG_EXPORT_PLCNAME]);
    dynAppend(dsOutput, dsParams[UN_CONFIG_EXPORT_APPLICATION]);

      // Get IEC104 connection settings
    sIecConnectionDp = "_IEC_" + dsParams[UN_CONFIG_EXPORT_PLCNAME];
    if( _unConfigIEC104_getConnectionSettings(sIecConnectionDp, mIecConnection, exceptionInfo) == FALSE )
    {
      fwException_raise(exceptionInfo, "ERROR", "IEC104_Com_ExportConfig() -> Error getting IEC104 connection data: " + sIecConnectionDp + ", due to: " + exceptionInfo[2], "");
      return;
    }

    fwGeneral_dynStringToString(mIecConnection[sIEC104_OBJ_CONN_LOCAL_LIST],  sLocal,  "|");
    fwGeneral_dynStringToString(mIecConnection[sIEC104_OBJ_CONN_GLOBAL_LIST], sGlobal, "|");

    iLen = dynlen(mIecConnection[sIEC104_OBJ_CONN_CONN_HOSTS]);
    for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
    {
      if( iLoop != 1 )
      {
        sHosts = sHosts + ",";
      }
      sHosts = sHosts + mIecConnection[sIEC104_OBJ_CONN_CONN_HOSTS][iLoop]    + "|" +
                        mIecConnection[sIEC104_OBJ_CONN_CONN_PORTS][iLoop]    + "|" +
                        mIecConnection[sIEC104_OBJ_CONN_CONN_TIME_OUT][iLoop];
    }

    dynAppend(dsOutput, sLocal);
    dynAppend(dsOutput, sGlobal);
    dynAppend(dsOutput, (bool)mIecConnection[sIEC104_OBJ_CONN_ACTIVE]);
    dynAppend(dsOutput, (bool)mIecConnection[sIEC104_OBJ_CONN_ENCRYPTION]);
    dynAppend(dsOutput, (bool)mIecConnection[sIEC104_OBJ_CONN_CHANGE_ROLE]);
    dynAppend(dsOutput,       mIecConnection[sIEC104_OBJ_CONN_INVALID_BIT]);
    dynAppend(dsOutput,       mIecConnection[sIEC104_OBJ_CONN_FORCE_ACTIVE]);
    dynAppend(dsOutput, sHosts);

    unExportDevice_writeDsStringToFile(dsOutput, UN_PARAMETER_DELIMITER);
  }
  else
  {
    fwException_raise(exceptionInfo, "ERROR", "IEC104_Com_ExportConfig() -> Invalid number of parameter in the export processL: " + iLen + ", and expected: " + UN_CONFIG_EXPORT_LENGTH, "");
    return;
  }

}//  IEC104_Com_ExportConfig()





/** Function checks whether addressReference follows the IEC104 address format.

  Address format: Reference|SubIndex|Convertion|Direction|COT

  Reference:                    Integers separated by dots: CommonAddress_HB.CommonAddress_LB.InformationObjectAddress_HB.InformationObjectAddress_MB.InformationObjectAddress_LB
  SubIndex:                     Integer,
  Direction:                    Integer, 2 = INPUT, 5 = Output individual, 1 = Output group
  ConvertionType:               Integer: from 1 to 242, check msg/en_US.utf8/iec.cat for details
  COT (Cause Of Transmission):  Integer, 1 = TRUE or 0 = FALSE

*/
bool unConfigIEC104_checkAddress(string sAddressReference , dyn_string &exceptionInfo)
{
  int iLoop, iLen, iValueRead, iRes;
  dyn_string dsAddress;


  // Check address field is empty
  if( sAddressReference == "" )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error address is empty. It should have following fields: Reference|SubIndex|ConversionType|Direction|COT", "");
    return FALSE;
  }


  // Check number of elements in the address field
  dsAddress = strsplit(sAddressReference, "|");
  iLen      = dynlen(dsAddress);
  if( iLen != iIEC104_ADDRESS_LENGTH )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error address: " + sAddressReference + " contains " + iLen + " fields and are expected: " + iIEC104_ADDRESS_LENGTH + ". It should have following fields: Reference|SubIndex|ConversionType|Direction|COT", "");
    return FALSE;
  }


  // Check the reference field


  // Check the subindex field
  unConfigGenericFunctions_checkInt(dsAddress[iIEC104_ADDRESS_SUBINDEX], iValueRead, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address subindex should be an integer value: " + exceptionInfo[2], "");
    return FALSE;
  }


  // Check the conversion type field
  unConfigGenericFunctions_checkInt(dsAddress[iIEC104_ADDRESS_DATATYPE], iValueRead, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address conversion type should be an integer value: " + exceptionInfo[2], "");
    return FALSE;
  }

  iRes = _unConfigIEC104_convertDataToType(dsAddress[iIEC104_ADDRESS_DATATYPE], FALSE, exceptionInfo);
  if( (iRes == -1) ||
      (iRes == 520)  )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address conversion value unknown: " + dsAddress[iIEC104_ADDRESS_DATATYPE], "");
    return FALSE;
  }


  // Check the direction field
  unConfigGenericFunctions_checkInt(dsAddress[iIEC104_ADDRESS_DIRECTION], iValueRead, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address direction should be an integer value: " + exceptionInfo[2], "");
    return FALSE;
  }

  switch( iValueRead )
  {
    case 1:
    case 2:
    case 5:
      break;

    default:
      fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address direction allowed values are only: 1, 2, 5. Current value is not allowed: " + dsAddress[iIEC104_ADDRESS_DIRECTION], "");
      return FALSE;
      break;
  }


  // Check the Cause Of Transmission field
  unConfigGenericFunctions_checkInt(dsAddress[iIEC104_ADDRESS_COT], iValueRead, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address COT value should be an integer value: " + exceptionInfo[2], "");
    return FALSE;
  }

  switch( iValueRead )
  {
    case 0:
    case 1:
      break;

    default:
      fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_checkAddress() -> Error, address COT allowed values are only: 0 and 1. Current value is not allowed: " + dsAddress[iIEC104_ADDRESS_COT], "");
      return FALSE;
      break;
  }

  return TRUE;

}//  unConfigIEC104_checkAddress()





/**
Purpose: Convert IEC-Typeident to _address.._datatype

Parameters:
      iIecData,    int, IEC type identifier 1-255

Return:  int, _address.._datatype conversion
*/
int _unConfigIEC104_convertDataToType(int iIecData, bool bCOT, dyn_string &exceptionInfo)
{
  if( bCOT )
  {
    return 544;
  }

  if( (iIecData == 0) ||
      (iIecData < 0)  ||
      (iIecData > 242) )
  {
    fwException_raise(exceptionInfo, "ERROR", "_unConfigIEC104_convertDataToType() -> Error Iec data type invalid should be in the range [1-242]", "");
    return -1;
  }


  switch( iIecData )
  {
    // IEC-Monitor
    case  1:      // M_SP_NA_1: Single-point information
    case  2:      // M_SP_TA_1: Single-point information with time tag
    case 30:      // M_SP_TB_1: Single-point information with time tag CP56Time2a
      return 521; // SIQ
      break;

    case  3:      // M_DP_NA_1: Double-point information
    case  4:      // M_DP_TA_1: Double-point information with time tag
    case 31:      // M_DP_TB_1: Double-point information with time tag CP56Time2a
      return 522; // DIQ
      break;

    case 5 :      // M_ST_NA_1: Step position information
    case 6 :      // M_ST_TA_1: Step position information with time tag
    case 32:      // M_ST_TB_1: Step position information with time tag CP56Time2a
      return 523; // VTI
      break;

    case 7 :      // M_BO_NA_1: Bitstring of 32 bit
    case 8 :      // M_BO_TA_1: Bitstring of 32 bit with time tag
    case 33:      // M_BO_TB_1: Step position information with time tag CP56Time2a
      return 531; // BSI
      break;

    case  9:      // M_ME_NA_1: Measured value, normalized value
    case 10:      // M_ME_TA_1: Measured value, normalized value with time tag
    case 34:      // M_ME_TD_1: Measured value, normalized value  with time tag CP56Time2a
      return 524; // NVA
      break;

    case 11:      // M_ME_NB_1: Measured value, scaled value
    case 12:      // M_ME_TB_1: Measured value, scaled value with time tag
    case 35:      // M_ME_TE_1: Measured value, scaled value with time tag CP56Time2a
      return 525; // SVA
      break;

    case 13:      // M_ME_NC_1: Measured value, short floating point value
    case 14:      // M_ME_TC_1: Measured value, short floating point value with time tag
    case 36:      // M_ME_TF_1: Measured value, short floating point value with time tag CP56Time2a
      return 526; // R32
      break;

    case 15:      // M_IT_NA_1: Integrated totals
    case 16:      // M_IT_TA_1: Integrated totals with time tag
    case 37:      // M_IT_TB_1: Integrated totals with time tag CP56Time2a
      return 527; // BCR
      break;

    case 17:      // M_EP_TA_1: Event of protection equipment with time tag
    case 38:      // M_EP_TD_1: Event of protection equipment with time tag CP56Time2a
      return 528; // SEP
      break;

    case 18:      // M_EP_TB_1: Packed start events of protection equipment with time tag
    case 39:      // M_EP_TE_1: Packed output circuit information of protection equipment with time tag with time tag CP56Time2a
      return 529; // SPE
      break;

    case 19:      // M_EP_TC_1: Packed output circuit information of protection equipment with time tag
    case 40:      // M_EP_TF_1: Packed single-point information with status change detection with time tag CP56Time2a
      return 530; // OCI
      break;

    case 20:      // M_PS_NA_1: Packed single-point information with status change detection
      return 531; // BSI preliminarry
      // return 535; // SCD
      break;

    case 21:      // M_ME_ND_1: Measured value, normalized value without quality descriptor
      return 524; // NVA
      break;


    // IEC Command
    case 45:      // C_SC_NA_1: Single command
    case 58:      // C_SC_TA_1: Single command with time tag CP56Time2a
      return 532; // SCO
      break;

    case 46:      // C_DC_NA_1: Double command
    case 59:      // C_DC_TA_1: Double command with time tag CP56Time2a
      return 533; // DCO
      break;

    case 47:      // C_RC_NA_1: Regulating step command
    case 60:      // C_RC_TA_1: Regulating step command with time tag CP56Time2a
      return 534; // RCO
      break;

    case 48:      // C_SE_NA_1: Set point command, normalized value
    case 61:      // C_SE_TA_1: Set point command, normalized value with time tag CP56Time2a
      return 524; // NVA
      break;

    case 49:      // C_SE_NB_1: Set point command, scaled value
    case 62:      // C_SE_TA_1: Set point command, scaled value with time tag CP56Time2a
      return 525; // SVA
      break;

    case 50:      // C_SE_NC_1: Set point command, short floating point value
    case 63:      // C_SE_TC_1: Set point command, short floating point value with time tag CP56Time2a
      return 526; // R32
      break;

    case 51:      // C_BO_NA_1: Bitstring of 32 bit
    case 64:      // C_BO_TA_1: Bitstring of 32 bit with time tag CP56Time2a
      return 531; // BSI
      break;


    // IEC End of Init
    case 70:      // M_EI_NA_1: End of initialization
      return 539; // Interrogation
      break;


    // IEC Special telegram
    case 100:     // C_IC_NA_1: Interrogation command
    case 101:     // C_CI_NA_1: Counter interrogation command
      return 539; // Interrogation
      break;

    case 102:     // C_RD_NA_1: Read command
      return 538; // Empty
      break;

    case 103:     // C_CS_NA_1: Clock synchronization command
      return 543; // CP56
      break;

    case 104:     // C_TS_NA_1: Check command
    case 107:     // C_TS_TA_1: Check command with Timestamp CP56Time2a
      return 540; // FBP
      break;

    case 105:     // C_RP_NA_1: Reset process command
      return 539; // Interrogation
      break;

    case 106:     // C_CD_NA_1: Delay acquisition command
      return 541; // CP16
      break;


    // IEC Parametrization
    case 110:     // P_ME_NA_1: Parameter of measured value, normalized value
      return 524; // NVA
      break;

    case 111:     // P_ME_NB_1: Parameter of measured value, scaled value
      return 525; // SVA
      break;

    case 112:     // P_MB_NC_1: Parameter of measured value, short floating point value
      return 526; // R32
      break;

    case 113:     // P_AC_NA_1: Parameter activation
      return 538; // Empty
      break;


    // SSI-Typen
    case 135:     // SSI System: System message
      return 537; // Private, Blob
      break;

    case 136:     // DF12: Double indication
      return 50;  // SsiTrans1DoubleIndType
      break;

    case 137:     // DF09: 16 single indications
      return 47;  // SsiTransSingleAlarmsType
      break;

    case 138:     // DF10: 8 double indications
      return 48;  // SsiTrans8DoubleIndType
      break;

    case 139:     // DF11: 4 double indications (command response)
      return 49;  // SsiTrans4DoubleIndType
      break;

    case 140:     // DF02: Digital value signed, 31 Bit + Vz
      return 42;  // SsiTransSignedInt32Type
      break;

    case 141:     // DF16: Bitstring 16 Bit
      return 47;  // SsiTransSingleAlarmsType (passt am besten)
      break;

    case 142:     // DF06: Transparent data
      return 52;  // SsiTransStrTranspType
      break;

    case 143:     // DF100: System message with time tag
      return 537; // Private, Blob
      break;

    case 160:     // DF13: Binary command with persistance time
      return 51;  // SsiTransBinaryCommandType
      break;


    // 8 octet integrated totals
    case 230:     // Measured value, 8 octet floating point integrated total
    case 231:     // Measured value, 8 octet floating point integrated total with time tag CP56Time2a
      return 546; // FBCR
      break;


    // Gasunie-Typen
    case 240:     // Integrated total 6 octet BCD
      return 545; // DBCR
      break;

    case 241:     // Integrated total 6 octet BCD with time tag CP56Time2a
      return 545; // DBCR
      break;

    case 242:     // Integrated total command 6 octet BCD
      return 545; // DBCR
      break;

    default:
      return 520;
  }
}






void unConfigIEC104_setAddress(string sDpe,
                               string sAddressReference,
                               int iMode,
                               int iDriverNumber,
                               int iDataType,
                               int iSubIndex = 0,
                               dyn_string& exceptionInfo)
{
  dyn_string dsParameters;


  if( sAddressReference == "" )
  {
    fwPeriphAddress_delete(sDpe, exceptionInfo);
    return;
  }
  else
  {
    dsParameters[FW_PARAMETER_FIELD_COMMUNICATION] = "IEC";
    dsParameters[FW_PARAMETER_FIELD_DRIVER]        = iDriverNumber;
    dsParameters[FW_PARAMETER_FIELD_ADDRESS]       = sAddressReference;
    dsParameters[FW_PARAMETER_FIELD_MODE]          = iMode;
    dsParameters[FW_PARAMETER_FIELD_DATATYPE]      = iDataType;
    dsParameters[FW_PARAMETER_FIELD_ACTIVE]        = 1;
    dsParameters[FW_PARAMETER_FIELD_SUBINDEX]      = iSubIndex;


    fwPeriphAddress_setIEC(sDpe, dsParameters, exceptionInfo);
    if( dynlen(exceptionInfo) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "unConfigIEC104_setAddress() -> Error configuring IEC address: " + sDpe + ", due to: " + exceptionInfo[2], "");
      return;
    }
  }

}//  unConfigIEC104_setAddress()






string DRV_IEC104_convertToUnicosAddress(string sDeviceDpeName, string sDeviceType)
{
  bool bConfigExists, bIsActive;
  int iLen, iDataType, iCot;
  string sAddress, sReference, sDataType;
  dyn_string exceptionInfo, dsSplit;
  dyn_anytype daConfig;


  // Get DPE address configuration
  fwPeriphAddress_get(sDeviceDpeName, bConfigExists, daConfig, bIsActive, exceptionInfo);
  if( dynlen(exceptionInfo) > 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "DRV_IEC104_convertToUnicosAddress() -> ERROR -> getting " + sDeviceDpeName + " address, due to: " + exceptionInfo[2], "");
    fwExceptionHandling_display(exceptionInfo);
    return "";
  }


  // Check if address is defined
  if( bConfigExists )
  {
    // Remove the IEC connection name from reference
    dsSplit = strsplit(daConfig[FW_PARAMETER_FIELD_ADDRESS], "-");
    iLen    = dynlen(dsSplit);
    if( iLen != 2 )
    {
      fwException_raise(exceptionInfo, "ERROR", "DRV_IEC104_convertToUnicosAddress() -> ERROR -> device " + sDeviceDpeName + " address reference should have only one - character: " + daConfig[FW_PARAMETER_FIELD_ADDRESS], "");
      fwExceptionHandling_display(exceptionInfo);
      return "";
    }

    // Remove the COT and data type from the reference
    sReference = dsSplit[2];
    dsSplit    = strsplit(sReference, ".");
    iLen       = dynlen(dsSplit);
    if( iLen != 6 )
    {
      fwException_raise(exceptionInfo, "ERROR", "DRV_IEC104_convertToUnicosAddress() -> ERROR -> device " + sDeviceDpeName + " address reference should have 6 values separated by dots: " + sReference, "");
      fwExceptionHandling_display(exceptionInfo);
      return "";
    }

    // Extract the data type and COT settings
    sDataType = dsSplit[1];
    if( (strpos(sDataType, "C") == 0)              &&
        (daConfig[fwPeriphAddress_DATATYPE] == 544)  )
    {
      iCot = 1;
      sscanf(sDataType, "C%d", iDataType);
    }
    else
    {
      iCot = 0;
      sscanf(sDataType, "%d", iDataType);
    }
    dynRemove(dsSplit, 1); // Remove COT and data type from address reference

    // Add the Common Address and Information Object Address fields to the final address reference
    fwGeneral_dynStringToString(dsSplit, sAddress, ".");

    // Add subindex to final address reference
    sAddress = sAddress + "|" + daConfig[FW_PARAMETER_FIELD_SUBINDEX];

    // Add data type to final address reference
    sAddress = sAddress + "|" + iDataType;

    // Add address direction to the final address reference
    sAddress = sAddress + "|" + daConfig[FW_PARAMETER_FIELD_MODE];

    // Add the COT to the final address reference
    sAddress = sAddress + "|" + iCot;
  }

  return sAddress;

}// DRV_IEC104_convertToUnicosAddress()
